// License: GPLv3 or later
// Author: AdmSasha <dik@inbox.ru>

#include "functions.h"
#include "cBuild.h"
#include <string>
#include <map>
#include <vector>
#include <pwd.h>

std::vector<std::string> srpms;

void show_usage() {
std::cout << _("\
======================================================================== \n\
     aum++ - RPM build tool for building in chroot \n\
======================================================================== \n\
\n\
Usage: aum++ [OPTIONS...] SRPM \n\
\n\
Options: \n \
\n\
General: \n\
\n\
    --help or -h           Show help \n\
    --chroot=PATH          Specify chroot tarball \n\
\n\
Distribs and medias: \n\
\n\
    --distrib=PATH         Use specified distrib \n\
    --distrib-own=PATH     Use specified own repo \n\
    --add-media=PATH       Add additional media \n\
\n\
Building: \n\
\n\
    --arch=ARCH            Specify arch \n\
    --log-file=FILE        Save logs in specified log file \n\
    --noclear              Don\'t remove build directory on exit \n\
    --timeout=SECONDS      Limit build process time (time for rpmbuild) \n\
    --tmp-dir=DIR          Build package in specified temp directory \n\
    --upload-dir=DIR       Upload dir \n\
    --upload-rpm-dir=DIR   Upload rpm dir \n\
    --upload-srpm-dir=DIR  Upload srpm dir \n\
    --add-rpmmacros=FILE   Change rpmmacros \n\
    --urpmi-options=WORD   Options for urpmi (see urpmi --help) \n\
    --norpmlint	           No check rpmlint \n\
\n\
Debugging: \n\
\n\
    --verbose or -v        Be more verbose \n\
\n\
    ") << std::endl;
}

map<string,string> loadArgvAum(int argc, char **argv){
    map<string,string> result;

    for (int i=1;i<argc;i++){
	string arg=string(argv[i]);

        boost::smatch xResults;	        
	if (boost::regex_match(arg,  xResults, boost::regex("(--)([-a-zA-Z0-9_]+)(=)(.*)"))==true){

	    //for (unsigned int a=0;a<xResults.size();a++) std::cout << xResults[a] << endl;

	    if (result[xResults[2]]==""){
		result[xResults[2]]=xResults[4];
	    }else{
		result[xResults[2]]+=";"+xResults[4];
	    }
	    continue;
	}
	if (boost::regex_match(arg,  xResults, boost::regex("([-]+)([-a-zA-Z0-9_]+)"))==true){
	    result[xResults[2]]="true";
	}
	if (boost::regex_match(arg,  xResults, boost::regex("^([^-])(.*)(\\.rpm)$"))==true){
	    srpms.push_back(arg);
	}
    }
    //result["srpm"]=string(argv[argc-1]);
    
    
    // Check for required parameters (Проверка на обязательные параметры)
    
    // Set defaults (Установка умолчаний)
    
    
    return result;
}

int main(int argc, char **argv){
    setlocale(LC_ALL, "");
//    char* cwd = getenv("PWD");

    if (boost::filesystem::exists("i18n")){
	bindtextdomain("aum++", "i18n");
    }else{
	bindtextdomain("aum++", "/usr/share/locale");
    }
    
    textdomain("aum++");

    if (getuid()!=0){
	std::cout << _("You must be root") << std::endl;
	return 1;
    }

    if (argc<2){
	show_usage();
	return 1;
    }
    if (strcmp(argv[1],"--help")==0 or strcmp(argv[1],"-h")==0){
	show_usage();
	return 1;
    }
    srand (time(NULL));

    map<string,string> options=loadArgvAum(argc,argv);

    if (srpms.size()==0){
	logdebug(_("Nothing to build"),options);
	return 1;
    }

    /*
    if (options["log-file"] !=""){
	std::fstream fs;
	fs.open(options["log-file"].c_str(), ios::out );
	fs.close();
    }
    */
    
    int returnCode=0;
    
    cBuild bb;
    for (unsigned int i=0;i<srpms.size();i++){
	options["srpm"]=srpms[i];
	
        logdebug(CharToString(_("Building of package %s was started"),options["srpm"].c_str()),options);
        
	if (bb.build(options)==0){
    	    logdebug(CharToString(_("Building of package %s is OK"),options["srpm"].c_str()),options);
	}else{
	    logdebug(CharToString(_("Building of package %s is fail"),options["srpm"].c_str()),options);
	    returnCode=1;
	}
    }

    return returnCode;
}
